# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal

import httpx

from ..types import text_to_speech_create_params
from .._types import Body, Query, Headers, NotGiven, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..lib.polling import (
    NewTaskCreatedResponse,
    AsyncNewTaskCreatedResponse,
    create_waitable_resource,
    create_async_waitable_resource,
)
from .._base_client import make_request_options
from ..types.text_to_speech_create_response import TextToSpeechCreateResponse

__all__ = ["TextToSpeechResource", "AsyncTextToSpeechResource"]


class TextToSpeechResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> TextToSpeechResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/runwayml/sdk-python#accessing-raw-response-data-eg-headers
        """
        return TextToSpeechResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> TextToSpeechResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/runwayml/sdk-python#with_streaming_response
        """
        return TextToSpeechResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        model: Literal["eleven_multilingual_v2"],
        prompt_text: str,
        voice: text_to_speech_create_params.Voice,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> NewTaskCreatedResponse:
        """
        This endpoint will start a new task to generate speech from text.

        Args:
          prompt_text: A non-empty string up to 1000 characters (measured in UTF-16 code units). This
              should describe in detail what should appear in the output.

          voice: A voice preset from the RunwayML API.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/v1/text_to_speech",
            body=maybe_transform(
                {
                    "model": model,
                    "prompt_text": prompt_text,
                    "voice": voice,
                },
                text_to_speech_create_params.TextToSpeechCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=create_waitable_resource(TextToSpeechCreateResponse, self._client),
        )


class AsyncTextToSpeechResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncTextToSpeechResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/runwayml/sdk-python#accessing-raw-response-data-eg-headers
        """
        return AsyncTextToSpeechResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncTextToSpeechResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/runwayml/sdk-python#with_streaming_response
        """
        return AsyncTextToSpeechResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        model: Literal["eleven_multilingual_v2"],
        prompt_text: str,
        voice: text_to_speech_create_params.Voice,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncNewTaskCreatedResponse:
        """
        This endpoint will start a new task to generate speech from text.

        Args:
          prompt_text: A non-empty string up to 1000 characters (measured in UTF-16 code units). This
              should describe in detail what should appear in the output.

          voice: A voice preset from the RunwayML API.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/v1/text_to_speech",
            body=await async_maybe_transform(
                {
                    "model": model,
                    "prompt_text": prompt_text,
                    "voice": voice,
                },
                text_to_speech_create_params.TextToSpeechCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=create_async_waitable_resource(TextToSpeechCreateResponse, self._client),
        )


class TextToSpeechResourceWithRawResponse:
    def __init__(self, text_to_speech: TextToSpeechResource) -> None:
        self._text_to_speech = text_to_speech

        self.create = to_raw_response_wrapper(
            text_to_speech.create,
        )


class AsyncTextToSpeechResourceWithRawResponse:
    def __init__(self, text_to_speech: AsyncTextToSpeechResource) -> None:
        self._text_to_speech = text_to_speech

        self.create = async_to_raw_response_wrapper(
            text_to_speech.create,
        )


class TextToSpeechResourceWithStreamingResponse:
    def __init__(self, text_to_speech: TextToSpeechResource) -> None:
        self._text_to_speech = text_to_speech

        self.create = to_streamed_response_wrapper(
            text_to_speech.create,
        )


class AsyncTextToSpeechResourceWithStreamingResponse:
    def __init__(self, text_to_speech: AsyncTextToSpeechResource) -> None:
        self._text_to_speech = text_to_speech

        self.create = async_to_streamed_response_wrapper(
            text_to_speech.create,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, Mapping
from typing_extensions import Self, override

import httpx

from . import _exceptions
from ._qs import Querystring
from ._types import (
    Omit,
    Timeout,
    NotGiven,
    Transport,
    ProxiesTypes,
    RequestOptions,
    not_given,
)
from ._utils import is_given, get_async_library
from ._version import __version__
from .resources import (
    tasks,
    uploads,
    organization,
    sound_effect,
    text_to_image,
    text_to_video,
    video_upscale,
    voice_dubbing,
    image_to_video,
    text_to_speech,
    video_to_video,
    voice_isolation,
    speech_to_speech,
    character_performance,
)
from ._streaming import Stream as Stream, AsyncStream as AsyncStream
from ._exceptions import RunwayMLError, APIStatusError
from ._base_client import (
    DEFAULT_MAX_RETRIES,
    SyncAPIClient,
    AsyncAPIClient,
)

__all__ = [
    "Timeout",
    "Transport",
    "ProxiesTypes",
    "RequestOptions",
    "RunwayML",
    "AsyncRunwayML",
    "Client",
    "AsyncClient",
]


class RunwayML(SyncAPIClient):
    tasks: tasks.TasksResource
    image_to_video: image_to_video.ImageToVideoResource
    video_to_video: video_to_video.VideoToVideoResource
    text_to_video: text_to_video.TextToVideoResource
    text_to_image: text_to_image.TextToImageResource
    video_upscale: video_upscale.VideoUpscaleResource
    character_performance: character_performance.CharacterPerformanceResource
    text_to_speech: text_to_speech.TextToSpeechResource
    sound_effect: sound_effect.SoundEffectResource
    voice_isolation: voice_isolation.VoiceIsolationResource
    voice_dubbing: voice_dubbing.VoiceDubbingResource
    speech_to_speech: speech_to_speech.SpeechToSpeechResource
    organization: organization.OrganizationResource
    uploads: uploads.UploadsResource
    with_raw_response: RunwayMLWithRawResponse
    with_streaming_response: RunwayMLWithStreamedResponse

    # client options
    api_key: str
    runway_version: str

    def __init__(
        self,
        *,
        api_key: str | None = None,
        runway_version: str | None = "2024-11-06",
        base_url: str | httpx.URL | None = None,
        timeout: float | Timeout | None | NotGiven = not_given,
        max_retries: int = DEFAULT_MAX_RETRIES,
        default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        # Configure a custom httpx client.
        # We provide a `DefaultHttpxClient` class that you can pass to retain the default values we use for `limits`, `timeout` & `follow_redirects`.
        # See the [httpx documentation](https://www.python-httpx.org/api/#client) for more details.
        http_client: httpx.Client | None = None,
        # Enable or disable schema validation for data returned by the API.
        # When enabled an error APIResponseValidationError is raised
        # if the API responds with invalid data for the expected schema.
        #
        # This parameter may be removed or changed in the future.
        # If you rely on this feature, please open a GitHub issue
        # outlining your use-case to help us decide if it should be
        # part of our public interface in the future.
        _strict_response_validation: bool = False,
    ) -> None:
        """Construct a new synchronous RunwayML client instance.

        This automatically infers the `api_key` argument from the `RUNWAYML_API_SECRET` environment variable if it is not provided.
        """
        if api_key is None:
            api_key = os.environ.get("RUNWAYML_API_SECRET")
        if api_key is None:
            raise RunwayMLError(
                "The api_key client option must be set either by passing api_key to the client or by setting the RUNWAYML_API_SECRET environment variable"
            )
        self.api_key = api_key

        if runway_version is None:
            runway_version = "2024-11-06"
        self.runway_version = runway_version

        if base_url is None:
            base_url = os.environ.get("RUNWAYML_BASE_URL")
        if base_url is None:
            base_url = f"https://api.dev.runwayml.com"

        super().__init__(
            version=__version__,
            base_url=base_url,
            max_retries=max_retries,
            timeout=timeout,
            http_client=http_client,
            custom_headers=default_headers,
            custom_query=default_query,
            _strict_response_validation=_strict_response_validation,
        )

        self.tasks = tasks.TasksResource(self)
        self.image_to_video = image_to_video.ImageToVideoResource(self)
        self.video_to_video = video_to_video.VideoToVideoResource(self)
        self.text_to_video = text_to_video.TextToVideoResource(self)
        self.text_to_image = text_to_image.TextToImageResource(self)
        self.video_upscale = video_upscale.VideoUpscaleResource(self)
        self.character_performance = character_performance.CharacterPerformanceResource(self)
        self.text_to_speech = text_to_speech.TextToSpeechResource(self)
        self.sound_effect = sound_effect.SoundEffectResource(self)
        self.voice_isolation = voice_isolation.VoiceIsolationResource(self)
        self.voice_dubbing = voice_dubbing.VoiceDubbingResource(self)
        self.speech_to_speech = speech_to_speech.SpeechToSpeechResource(self)
        self.organization = organization.OrganizationResource(self)
        self.uploads = uploads.UploadsResource(self)
        self.with_raw_response = RunwayMLWithRawResponse(self)
        self.with_streaming_response = RunwayMLWithStreamedResponse(self)

    @property
    @override
    def qs(self) -> Querystring:
        return Querystring(array_format="comma")

    @property
    @override
    def auth_headers(self) -> dict[str, str]:
        api_key = self.api_key
        return {"Authorization": f"Bearer {api_key}"}

    @property
    @override
    def default_headers(self) -> dict[str, str | Omit]:
        return {
            **super().default_headers,
            "X-Stainless-Async": "false",
            "X-Runway-Version": self.runway_version,
            **self._custom_headers,
        }

    def copy(
        self,
        *,
        api_key: str | None = None,
        runway_version: str | None = None,
        base_url: str | httpx.URL | None = None,
        timeout: float | Timeout | None | NotGiven = not_given,
        http_client: httpx.Client | None = None,
        max_retries: int | NotGiven = not_given,
        default_headers: Mapping[str, str] | None = None,
        set_default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        set_default_query: Mapping[str, object] | None = None,
        _extra_kwargs: Mapping[str, Any] = {},
    ) -> Self:
        """
        Create a new client instance re-using the same options given to the current client with optional overriding.
        """
        if default_headers is not None and set_default_headers is not None:
            raise ValueError("The `default_headers` and `set_default_headers` arguments are mutually exclusive")

        if default_query is not None and set_default_query is not None:
            raise ValueError("The `default_query` and `set_default_query` arguments are mutually exclusive")

        headers = self._custom_headers
        if default_headers is not None:
            headers = {**headers, **default_headers}
        elif set_default_headers is not None:
            headers = set_default_headers

        params = self._custom_query
        if default_query is not None:
            params = {**params, **default_query}
        elif set_default_query is not None:
            params = set_default_query

        http_client = http_client or self._client
        return self.__class__(
            api_key=api_key or self.api_key,
            runway_version=runway_version or self.runway_version,
            base_url=base_url or self.base_url,
            timeout=self.timeout if isinstance(timeout, NotGiven) else timeout,
            http_client=http_client,
            max_retries=max_retries if is_given(max_retries) else self.max_retries,
            default_headers=headers,
            default_query=params,
            **_extra_kwargs,
        )

    # Alias for `copy` for nicer inline usage, e.g.
    # client.with_options(timeout=10).foo.create(...)
    with_options = copy

    @override
    def _make_status_error(
        self,
        err_msg: str,
        *,
        body: object,
        response: httpx.Response,
    ) -> APIStatusError:
        if response.status_code == 400:
            return _exceptions.BadRequestError(err_msg, response=response, body=body)

        if response.status_code == 401:
            return _exceptions.AuthenticationError(err_msg, response=response, body=body)

        if response.status_code == 403:
            return _exceptions.PermissionDeniedError(err_msg, response=response, body=body)

        if response.status_code == 404:
            return _exceptions.NotFoundError(err_msg, response=response, body=body)

        if response.status_code == 409:
            return _exceptions.ConflictError(err_msg, response=response, body=body)

        if response.status_code == 422:
            return _exceptions.UnprocessableEntityError(err_msg, response=response, body=body)

        if response.status_code == 429:
            return _exceptions.RateLimitError(err_msg, response=response, body=body)

        if response.status_code >= 500:
            return _exceptions.InternalServerError(err_msg, response=response, body=body)
        return APIStatusError(err_msg, response=response, body=body)


class AsyncRunwayML(AsyncAPIClient):
    tasks: tasks.AsyncTasksResource
    image_to_video: image_to_video.AsyncImageToVideoResource
    video_to_video: video_to_video.AsyncVideoToVideoResource
    text_to_video: text_to_video.AsyncTextToVideoResource
    text_to_image: text_to_image.AsyncTextToImageResource
    video_upscale: video_upscale.AsyncVideoUpscaleResource
    character_performance: character_performance.AsyncCharacterPerformanceResource
    text_to_speech: text_to_speech.AsyncTextToSpeechResource
    sound_effect: sound_effect.AsyncSoundEffectResource
    voice_isolation: voice_isolation.AsyncVoiceIsolationResource
    voice_dubbing: voice_dubbing.AsyncVoiceDubbingResource
    speech_to_speech: speech_to_speech.AsyncSpeechToSpeechResource
    organization: organization.AsyncOrganizationResource
    uploads: uploads.AsyncUploadsResource
    with_raw_response: AsyncRunwayMLWithRawResponse
    with_streaming_response: AsyncRunwayMLWithStreamedResponse

    # client options
    api_key: str
    runway_version: str

    def __init__(
        self,
        *,
        api_key: str | None = None,
        runway_version: str | None = "2024-11-06",
        base_url: str | httpx.URL | None = None,
        timeout: float | Timeout | None | NotGiven = not_given,
        max_retries: int = DEFAULT_MAX_RETRIES,
        default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        # Configure a custom httpx client.
        # We provide a `DefaultAsyncHttpxClient` class that you can pass to retain the default values we use for `limits`, `timeout` & `follow_redirects`.
        # See the [httpx documentation](https://www.python-httpx.org/api/#asyncclient) for more details.
        http_client: httpx.AsyncClient | None = None,
        # Enable or disable schema validation for data returned by the API.
        # When enabled an error APIResponseValidationError is raised
        # if the API responds with invalid data for the expected schema.
        #
        # This parameter may be removed or changed in the future.
        # If you rely on this feature, please open a GitHub issue
        # outlining your use-case to help us decide if it should be
        # part of our public interface in the future.
        _strict_response_validation: bool = False,
    ) -> None:
        """Construct a new async AsyncRunwayML client instance.

        This automatically infers the `api_key` argument from the `RUNWAYML_API_SECRET` environment variable if it is not provided.
        """
        if api_key is None:
            api_key = os.environ.get("RUNWAYML_API_SECRET")
        if api_key is None:
            raise RunwayMLError(
                "The api_key client option must be set either by passing api_key to the client or by setting the RUNWAYML_API_SECRET environment variable"
            )
        self.api_key = api_key

        if runway_version is None:
            runway_version = "2024-11-06"
        self.runway_version = runway_version

        if base_url is None:
            base_url = os.environ.get("RUNWAYML_BASE_URL")
        if base_url is None:
            base_url = f"https://api.dev.runwayml.com"

        super().__init__(
            version=__version__,
            base_url=base_url,
            max_retries=max_retries,
            timeout=timeout,
            http_client=http_client,
            custom_headers=default_headers,
            custom_query=default_query,
            _strict_response_validation=_strict_response_validation,
        )

        self.tasks = tasks.AsyncTasksResource(self)
        self.image_to_video = image_to_video.AsyncImageToVideoResource(self)
        self.video_to_video = video_to_video.AsyncVideoToVideoResource(self)
        self.text_to_video = text_to_video.AsyncTextToVideoResource(self)
        self.text_to_image = text_to_image.AsyncTextToImageResource(self)
        self.video_upscale = video_upscale.AsyncVideoUpscaleResource(self)
        self.character_performance = character_performance.AsyncCharacterPerformanceResource(self)
        self.text_to_speech = text_to_speech.AsyncTextToSpeechResource(self)
        self.sound_effect = sound_effect.AsyncSoundEffectResource(self)
        self.voice_isolation = voice_isolation.AsyncVoiceIsolationResource(self)
        self.voice_dubbing = voice_dubbing.AsyncVoiceDubbingResource(self)
        self.speech_to_speech = speech_to_speech.AsyncSpeechToSpeechResource(self)
        self.organization = organization.AsyncOrganizationResource(self)
        self.uploads = uploads.AsyncUploadsResource(self)
        self.with_raw_response = AsyncRunwayMLWithRawResponse(self)
        self.with_streaming_response = AsyncRunwayMLWithStreamedResponse(self)

    @property
    @override
    def qs(self) -> Querystring:
        return Querystring(array_format="comma")

    @property
    @override
    def auth_headers(self) -> dict[str, str]:
        api_key = self.api_key
        return {"Authorization": f"Bearer {api_key}"}

    @property
    @override
    def default_headers(self) -> dict[str, str | Omit]:
        return {
            **super().default_headers,
            "X-Stainless-Async": f"async:{get_async_library()}",
            "X-Runway-Version": self.runway_version,
            **self._custom_headers,
        }

    def copy(
        self,
        *,
        api_key: str | None = None,
        runway_version: str | None = None,
        base_url: str | httpx.URL | None = None,
        timeout: float | Timeout | None | NotGiven = not_given,
        http_client: httpx.AsyncClient | None = None,
        max_retries: int | NotGiven = not_given,
        default_headers: Mapping[str, str] | None = None,
        set_default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        set_default_query: Mapping[str, object] | None = None,
        _extra_kwargs: Mapping[str, Any] = {},
    ) -> Self:
        """
        Create a new client instance re-using the same options given to the current client with optional overriding.
        """
        if default_headers is not None and set_default_headers is not None:
            raise ValueError("The `default_headers` and `set_default_headers` arguments are mutually exclusive")

        if default_query is not None and set_default_query is not None:
            raise ValueError("The `default_query` and `set_default_query` arguments are mutually exclusive")

        headers = self._custom_headers
        if default_headers is not None:
            headers = {**headers, **default_headers}
        elif set_default_headers is not None:
            headers = set_default_headers

        params = self._custom_query
        if default_query is not None:
            params = {**params, **default_query}
        elif set_default_query is not None:
            params = set_default_query

        http_client = http_client or self._client
        return self.__class__(
            api_key=api_key or self.api_key,
            runway_version=runway_version or self.runway_version,
            base_url=base_url or self.base_url,
            timeout=self.timeout if isinstance(timeout, NotGiven) else timeout,
            http_client=http_client,
            max_retries=max_retries if is_given(max_retries) else self.max_retries,
            default_headers=headers,
            default_query=params,
            **_extra_kwargs,
        )

    # Alias for `copy` for nicer inline usage, e.g.
    # client.with_options(timeout=10).foo.create(...)
    with_options = copy

    @override
    def _make_status_error(
        self,
        err_msg: str,
        *,
        body: object,
        response: httpx.Response,
    ) -> APIStatusError:
        if response.status_code == 400:
            return _exceptions.BadRequestError(err_msg, response=response, body=body)

        if response.status_code == 401:
            return _exceptions.AuthenticationError(err_msg, response=response, body=body)

        if response.status_code == 403:
            return _exceptions.PermissionDeniedError(err_msg, response=response, body=body)

        if response.status_code == 404:
            return _exceptions.NotFoundError(err_msg, response=response, body=body)

        if response.status_code == 409:
            return _exceptions.ConflictError(err_msg, response=response, body=body)

        if response.status_code == 422:
            return _exceptions.UnprocessableEntityError(err_msg, response=response, body=body)

        if response.status_code == 429:
            return _exceptions.RateLimitError(err_msg, response=response, body=body)

        if response.status_code >= 500:
            return _exceptions.InternalServerError(err_msg, response=response, body=body)
        return APIStatusError(err_msg, response=response, body=body)


class RunwayMLWithRawResponse:
    def __init__(self, client: RunwayML) -> None:
        self.tasks = tasks.TasksResourceWithRawResponse(client.tasks)
        self.image_to_video = image_to_video.ImageToVideoResourceWithRawResponse(client.image_to_video)
        self.video_to_video = video_to_video.VideoToVideoResourceWithRawResponse(client.video_to_video)
        self.text_to_video = text_to_video.TextToVideoResourceWithRawResponse(client.text_to_video)
        self.text_to_image = text_to_image.TextToImageResourceWithRawResponse(client.text_to_image)
        self.video_upscale = video_upscale.VideoUpscaleResourceWithRawResponse(client.video_upscale)
        self.character_performance = character_performance.CharacterPerformanceResourceWithRawResponse(
            client.character_performance
        )
        self.text_to_speech = text_to_speech.TextToSpeechResourceWithRawResponse(client.text_to_speech)
        self.sound_effect = sound_effect.SoundEffectResourceWithRawResponse(client.sound_effect)
        self.voice_isolation = voice_isolation.VoiceIsolationResourceWithRawResponse(client.voice_isolation)
        self.voice_dubbing = voice_dubbing.VoiceDubbingResourceWithRawResponse(client.voice_dubbing)
        self.speech_to_speech = speech_to_speech.SpeechToSpeechResourceWithRawResponse(client.speech_to_speech)
        self.organization = organization.OrganizationResourceWithRawResponse(client.organization)
        self.uploads = uploads.UploadsResourceWithRawResponse(client.uploads)


class AsyncRunwayMLWithRawResponse:
    def __init__(self, client: AsyncRunwayML) -> None:
        self.tasks = tasks.AsyncTasksResourceWithRawResponse(client.tasks)
        self.image_to_video = image_to_video.AsyncImageToVideoResourceWithRawResponse(client.image_to_video)
        self.video_to_video = video_to_video.AsyncVideoToVideoResourceWithRawResponse(client.video_to_video)
        self.text_to_video = text_to_video.AsyncTextToVideoResourceWithRawResponse(client.text_to_video)
        self.text_to_image = text_to_image.AsyncTextToImageResourceWithRawResponse(client.text_to_image)
        self.video_upscale = video_upscale.AsyncVideoUpscaleResourceWithRawResponse(client.video_upscale)
        self.character_performance = character_performance.AsyncCharacterPerformanceResourceWithRawResponse(
            client.character_performance
        )
        self.text_to_speech = text_to_speech.AsyncTextToSpeechResourceWithRawResponse(client.text_to_speech)
        self.sound_effect = sound_effect.AsyncSoundEffectResourceWithRawResponse(client.sound_effect)
        self.voice_isolation = voice_isolation.AsyncVoiceIsolationResourceWithRawResponse(client.voice_isolation)
        self.voice_dubbing = voice_dubbing.AsyncVoiceDubbingResourceWithRawResponse(client.voice_dubbing)
        self.speech_to_speech = speech_to_speech.AsyncSpeechToSpeechResourceWithRawResponse(client.speech_to_speech)
        self.organization = organization.AsyncOrganizationResourceWithRawResponse(client.organization)
        self.uploads = uploads.AsyncUploadsResourceWithRawResponse(client.uploads)


class RunwayMLWithStreamedResponse:
    def __init__(self, client: RunwayML) -> None:
        self.tasks = tasks.TasksResourceWithStreamingResponse(client.tasks)
        self.image_to_video = image_to_video.ImageToVideoResourceWithStreamingResponse(client.image_to_video)
        self.video_to_video = video_to_video.VideoToVideoResourceWithStreamingResponse(client.video_to_video)
        self.text_to_video = text_to_video.TextToVideoResourceWithStreamingResponse(client.text_to_video)
        self.text_to_image = text_to_image.TextToImageResourceWithStreamingResponse(client.text_to_image)
        self.video_upscale = video_upscale.VideoUpscaleResourceWithStreamingResponse(client.video_upscale)
        self.character_performance = character_performance.CharacterPerformanceResourceWithStreamingResponse(
            client.character_performance
        )
        self.text_to_speech = text_to_speech.TextToSpeechResourceWithStreamingResponse(client.text_to_speech)
        self.sound_effect = sound_effect.SoundEffectResourceWithStreamingResponse(client.sound_effect)
        self.voice_isolation = voice_isolation.VoiceIsolationResourceWithStreamingResponse(client.voice_isolation)
        self.voice_dubbing = voice_dubbing.VoiceDubbingResourceWithStreamingResponse(client.voice_dubbing)
        self.speech_to_speech = speech_to_speech.SpeechToSpeechResourceWithStreamingResponse(client.speech_to_speech)
        self.organization = organization.OrganizationResourceWithStreamingResponse(client.organization)
        self.uploads = uploads.UploadsResourceWithStreamingResponse(client.uploads)


class AsyncRunwayMLWithStreamedResponse:
    def __init__(self, client: AsyncRunwayML) -> None:
        self.tasks = tasks.AsyncTasksResourceWithStreamingResponse(client.tasks)
        self.image_to_video = image_to_video.AsyncImageToVideoResourceWithStreamingResponse(client.image_to_video)
        self.video_to_video = video_to_video.AsyncVideoToVideoResourceWithStreamingResponse(client.video_to_video)
        self.text_to_video = text_to_video.AsyncTextToVideoResourceWithStreamingResponse(client.text_to_video)
        self.text_to_image = text_to_image.AsyncTextToImageResourceWithStreamingResponse(client.text_to_image)
        self.video_upscale = video_upscale.AsyncVideoUpscaleResourceWithStreamingResponse(client.video_upscale)
        self.character_performance = character_performance.AsyncCharacterPerformanceResourceWithStreamingResponse(
            client.character_performance
        )
        self.text_to_speech = text_to_speech.AsyncTextToSpeechResourceWithStreamingResponse(client.text_to_speech)
        self.sound_effect = sound_effect.AsyncSoundEffectResourceWithStreamingResponse(client.sound_effect)
        self.voice_isolation = voice_isolation.AsyncVoiceIsolationResourceWithStreamingResponse(client.voice_isolation)
        self.voice_dubbing = voice_dubbing.AsyncVoiceDubbingResourceWithStreamingResponse(client.voice_dubbing)
        self.speech_to_speech = speech_to_speech.AsyncSpeechToSpeechResourceWithStreamingResponse(
            client.speech_to_speech
        )
        self.organization = organization.AsyncOrganizationResourceWithStreamingResponse(client.organization)
        self.uploads = uploads.AsyncUploadsResourceWithStreamingResponse(client.uploads)


Client = RunwayML

AsyncClient = AsyncRunwayML



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from runwayml import RunwayML, AsyncRunwayML
from tests.utils import assert_matches_type
from runwayml.types import CharacterPerformanceCreateResponse

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestCharacterPerformance:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_method_create(self, client: RunwayML) -> None:
        character_performance = client.character_performance.create(
            character={
                "type": "image",
                "uri": "https://example.com/file",
            },
            model="act_two",
            reference={
                "type": "video",
                "uri": "https://example.com/reference-performance.mp4",
            },
        )
        assert_matches_type(CharacterPerformanceCreateResponse, character_performance, path=["response"])

    @parametrize
    def test_method_create_with_all_params(self, client: RunwayML) -> None:
        character_performance = client.character_performance.create(
            character={
                "type": "image",
                "uri": "https://example.com/file",
            },
            model="act_two",
            reference={
                "type": "video",
                "uri": "https://example.com/reference-performance.mp4",
            },
            body_control=True,
            content_moderation={"public_figure_threshold": "auto"},
            expression_intensity=3,
            ratio="1280:720",
            seed=0,
        )
        assert_matches_type(CharacterPerformanceCreateResponse, character_performance, path=["response"])

    @parametrize
    def test_raw_response_create(self, client: RunwayML) -> None:
        response = client.character_performance.with_raw_response.create(
            character={
                "type": "image",
                "uri": "https://example.com/file",
            },
            model="act_two",
            reference={
                "type": "video",
                "uri": "https://example.com/reference-performance.mp4",
            },
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        character_performance = response.parse()
        assert_matches_type(CharacterPerformanceCreateResponse, character_performance, path=["response"])

    @parametrize
    def test_streaming_response_create(self, client: RunwayML) -> None:
        with client.character_performance.with_streaming_response.create(
            character={
                "type": "image",
                "uri": "https://example.com/file",
            },
            model="act_two",
            reference={
                "type": "video",
                "uri": "https://example.com/reference-performance.mp4",
            },
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            character_performance = response.parse()
            assert_matches_type(CharacterPerformanceCreateResponse, character_performance, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncCharacterPerformance:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @parametrize
    async def test_method_create(self, async_client: AsyncRunwayML) -> None:
        character_performance = await async_client.character_performance.create(
            character={
                "type": "image",
                "uri": "https://example.com/file",
            },
            model="act_two",
            reference={
                "type": "video",
                "uri": "https://example.com/reference-performance.mp4",
            },
        )
        assert_matches_type(CharacterPerformanceCreateResponse, character_performance, path=["response"])

    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncRunwayML) -> None:
        character_performance = await async_client.character_performance.create(
            character={
                "type": "image",
                "uri": "https://example.com/file",
            },
            model="act_two",
            reference={
                "type": "video",
                "uri": "https://example.com/reference-performance.mp4",
            },
            body_control=True,
            content_moderation={"public_figure_threshold": "auto"},
            expression_intensity=3,
            ratio="1280:720",
            seed=0,
        )
        assert_matches_type(CharacterPerformanceCreateResponse, character_performance, path=["response"])

    @parametrize
    async def test_raw_response_create(self, async_client: AsyncRunwayML) -> None:
        response = await async_client.character_performance.with_raw_response.create(
            character={
                "type": "image",
                "uri": "https://example.com/file",
            },
            model="act_two",
            reference={
                "type": "video",
                "uri": "https://example.com/reference-performance.mp4",
            },
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        character_performance = await response.parse()
        assert_matches_type(CharacterPerformanceCreateResponse, character_performance, path=["response"])

    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncRunwayML) -> None:
        async with async_client.character_performance.with_streaming_response.create(
            character={
                "type": "image",
                "uri": "https://example.com/file",
            },
            model="act_two",
            reference={
                "type": "video",
                "uri": "https://example.com/reference-performance.mp4",
            },
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            character_performance = await response.parse()
            assert_matches_type(CharacterPerformanceCreateResponse, character_performance, path=["response"])

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from pydantic import Field as FieldInfo

from .._models import BaseModel

__all__ = [
    "OrganizationRetrieveResponse",
    "Tier",
    "TierModels",
    "TierModelsActTwo",
    "TierModelsElevenMultilingualStsV2",
    "TierModelsElevenMultilingualV2",
    "TierModelsElevenTextToSoundV2",
    "TierModelsElevenVoiceDubbing",
    "TierModelsElevenVoiceIsolation",
    "TierModelsGemini2_5Flash",
    "TierModelsGen3aTurbo",
    "TierModelsGen4Aleph",
    "TierModelsGen4Image",
    "TierModelsGen4ImageTurbo",
    "TierModelsGen4Turbo",
    "TierModelsUpscaleV1",
    "TierModelsVeo3",
    "TierModelsVeo3_1",
    "TierModelsVeo3_1Fast",
    "Usage",
    "UsageModels",
    "UsageModelsActTwo",
    "UsageModelsElevenMultilingualStsV2",
    "UsageModelsElevenMultilingualV2",
    "UsageModelsElevenTextToSoundV2",
    "UsageModelsElevenVoiceDubbing",
    "UsageModelsElevenVoiceIsolation",
    "UsageModelsGemini2_5Flash",
    "UsageModelsGen3aTurbo",
    "UsageModelsGen4Aleph",
    "UsageModelsGen4Image",
    "UsageModelsGen4ImageTurbo",
    "UsageModelsGen4Turbo",
    "UsageModelsUpscaleV1",
    "UsageModelsVeo3",
    "UsageModelsVeo3_1",
    "UsageModelsVeo3_1Fast",
]


class TierModelsActTwo(BaseModel):
    max_concurrent_generations: int = FieldInfo(alias="maxConcurrentGenerations")
    """The maximum number of generations that can be run concurrently for this model."""

    max_daily_generations: int = FieldInfo(alias="maxDailyGenerations")
    """The maximum number of generations that can be created each day for this model."""


class TierModelsElevenMultilingualStsV2(BaseModel):
    max_concurrent_generations: int = FieldInfo(alias="maxConcurrentGenerations")
    """The maximum number of generations that can be run concurrently for this model."""

    max_daily_generations: int = FieldInfo(alias="maxDailyGenerations")
    """The maximum number of generations that can be created each day for this model."""


class TierModelsElevenMultilingualV2(BaseModel):
    max_concurrent_generations: int = FieldInfo(alias="maxConcurrentGenerations")
    """The maximum number of generations that can be run concurrently for this model."""

    max_daily_generations: int = FieldInfo(alias="maxDailyGenerations")
    """The maximum number of generations that can be created each day for this model."""


class TierModelsElevenTextToSoundV2(BaseModel):
    max_concurrent_generations: int = FieldInfo(alias="maxConcurrentGenerations")
    """The maximum number of generations that can be run concurrently for this model."""

    max_daily_generations: int = FieldInfo(alias="maxDailyGenerations")
    """The maximum number of generations that can be created each day for this model."""


class TierModelsElevenVoiceDubbing(BaseModel):
    max_concurrent_generations: int = FieldInfo(alias="maxConcurrentGenerations")
    """The maximum number of generations that can be run concurrently for this model."""

    max_daily_generations: int = FieldInfo(alias="maxDailyGenerations")
    """The maximum number of generations that can be created each day for this model."""


class TierModelsElevenVoiceIsolation(BaseModel):
    max_concurrent_generations: int = FieldInfo(alias="maxConcurrentGenerations")
    """The maximum number of generations that can be run concurrently for this model."""

    max_daily_generations: int = FieldInfo(alias="maxDailyGenerations")
    """The maximum number of generations that can be created each day for this model."""


class TierModelsGemini2_5Flash(BaseModel):
    max_concurrent_generations: int = FieldInfo(alias="maxConcurrentGenerations")
    """The maximum number of generations that can be run concurrently for this model."""

    max_daily_generations: int = FieldInfo(alias="maxDailyGenerations")
    """The maximum number of generations that can be created each day for this model."""


class TierModelsGen3aTurbo(BaseModel):
    max_concurrent_generations: int = FieldInfo(alias="maxConcurrentGenerations")
    """The maximum number of generations that can be run concurrently for this model."""

    max_daily_generations: int = FieldInfo(alias="maxDailyGenerations")
    """The maximum number of generations that can be created each day for this model."""


class TierModelsGen4Aleph(BaseModel):
    max_concurrent_generations: int = FieldInfo(alias="maxConcurrentGenerations")
    """The maximum number of generations that can be run concurrently for this model."""

    max_daily_generations: int = FieldInfo(alias="maxDailyGenerations")
    """The maximum number of generations that can be created each day for this model."""


class TierModelsGen4Image(BaseModel):
    max_concurrent_generations: int = FieldInfo(alias="maxConcurrentGenerations")
    """The maximum number of generations that can be run concurrently for this model."""

    max_daily_generations: int = FieldInfo(alias="maxDailyGenerations")
    """The maximum number of generations that can be created each day for this model."""


class TierModelsGen4ImageTurbo(BaseModel):
    max_concurrent_generations: int = FieldInfo(alias="maxConcurrentGenerations")
    """The maximum number of generations that can be run concurrently for this model."""

    max_daily_generations: int = FieldInfo(alias="maxDailyGenerations")
    """The maximum number of generations that can be created each day for this model."""


class TierModelsGen4Turbo(BaseModel):
    max_concurrent_generations: int = FieldInfo(alias="maxConcurrentGenerations")
    """The maximum number of generations that can be run concurrently for this model."""

    max_daily_generations: int = FieldInfo(alias="maxDailyGenerations")
    """The maximum number of generations that can be created each day for this model."""


class TierModelsUpscaleV1(BaseModel):
    max_concurrent_generations: int = FieldInfo(alias="maxConcurrentGenerations")
    """The maximum number of generations that can be run concurrently for this model."""

    max_daily_generations: int = FieldInfo(alias="maxDailyGenerations")
    """The maximum number of generations that can be created each day for this model."""


class TierModelsVeo3(BaseModel):
    max_concurrent_generations: int = FieldInfo(alias="maxConcurrentGenerations")
    """The maximum number of generations that can be run concurrently for this model."""

    max_daily_generations: int = FieldInfo(alias="maxDailyGenerations")
    """The maximum number of generations that can be created each day for this model."""


class TierModelsVeo3_1(BaseModel):
    max_concurrent_generations: int = FieldInfo(alias="maxConcurrentGenerations")
    """The maximum number of generations that can be run concurrently for this model."""

    max_daily_generations: int = FieldInfo(alias="maxDailyGenerations")
    """The maximum number of generations that can be created each day for this model."""


class TierModelsVeo3_1Fast(BaseModel):
    max_concurrent_generations: int = FieldInfo(alias="maxConcurrentGenerations")
    """The maximum number of generations that can be run concurrently for this model."""

    max_daily_generations: int = FieldInfo(alias="maxDailyGenerations")
    """The maximum number of generations that can be created each day for this model."""


class TierModels(BaseModel):
    act_two: Optional[TierModelsActTwo] = None
    """Limits associated with the act_two model."""

    eleven_multilingual_sts_v2: Optional[TierModelsElevenMultilingualStsV2] = None
    """Limits associated with the eleven_multilingual_sts_v2 model."""

    eleven_multilingual_v2: Optional[TierModelsElevenMultilingualV2] = None
    """Limits associated with the eleven_multilingual_v2 model."""

    eleven_text_to_sound_v2: Optional[TierModelsElevenTextToSoundV2] = None
    """Limits associated with the eleven_text_to_sound_v2 model."""

    eleven_voice_dubbing: Optional[TierModelsElevenVoiceDubbing] = None
    """Limits associated with the eleven_voice_dubbing model."""

    eleven_voice_isolation: Optional[TierModelsElevenVoiceIsolation] = None
    """Limits associated with the eleven_voice_isolation model."""

    gemini_2_5_flash: Optional[TierModelsGemini2_5Flash] = FieldInfo(alias="gemini_2.5_flash", default=None)
    """Limits associated with the gemini_2.5_flash model."""

    gen3a_turbo: Optional[TierModelsGen3aTurbo] = None
    """Limits associated with the gen3a_turbo model."""

    gen4_aleph: Optional[TierModelsGen4Aleph] = None
    """Limits associated with the gen4_aleph model."""

    gen4_image: Optional[TierModelsGen4Image] = None
    """Limits associated with the gen4_image model."""

    gen4_image_turbo: Optional[TierModelsGen4ImageTurbo] = None
    """Limits associated with the gen4_image_turbo model."""

    gen4_turbo: Optional[TierModelsGen4Turbo] = None
    """Limits associated with the gen4_turbo model."""

    upscale_v1: Optional[TierModelsUpscaleV1] = None
    """Limits associated with the upscale_v1 model."""

    veo3: Optional[TierModelsVeo3] = None
    """Limits associated with the veo3 model."""

    veo3_1: Optional[TierModelsVeo3_1] = FieldInfo(alias="veo3.1", default=None)
    """Limits associated with the veo3.1 model."""

    veo3_1_fast: Optional[TierModelsVeo3_1Fast] = FieldInfo(alias="veo3.1_fast", default=None)
    """Limits associated with the veo3.1_fast model."""


class Tier(BaseModel):
    max_monthly_credit_spend: int = FieldInfo(alias="maxMonthlyCreditSpend")
    """The maximum number of credits that can be purchased in a month."""

    models: TierModels
    """An object containing model-specific limits. Each key represents a model."""


class UsageModelsActTwo(BaseModel):
    daily_generations: int = FieldInfo(alias="dailyGenerations")
    """The number of generations that have been run for this model in the past day."""


class UsageModelsElevenMultilingualStsV2(BaseModel):
    daily_generations: int = FieldInfo(alias="dailyGenerations")
    """The number of generations that have been run for this model in the past day."""


class UsageModelsElevenMultilingualV2(BaseModel):
    daily_generations: int = FieldInfo(alias="dailyGenerations")
    """The number of generations that have been run for this model in the past day."""


class UsageModelsElevenTextToSoundV2(BaseModel):
    daily_generations: int = FieldInfo(alias="dailyGenerations")
    """The number of generations that have been run for this model in the past day."""


class UsageModelsElevenVoiceDubbing(BaseModel):
    daily_generations: int = FieldInfo(alias="dailyGenerations")
    """The number of generations that have been run for this model in the past day."""


class UsageModelsElevenVoiceIsolation(BaseModel):
    daily_generations: int = FieldInfo(alias="dailyGenerations")
    """The number of generations that have been run for this model in the past day."""


class UsageModelsGemini2_5Flash(BaseModel):
    daily_generations: int = FieldInfo(alias="dailyGenerations")
    """The number of generations that have been run for this model in the past day."""


class UsageModelsGen3aTurbo(BaseModel):
    daily_generations: int = FieldInfo(alias="dailyGenerations")
    """The number of generations that have been run for this model in the past day."""


class UsageModelsGen4Aleph(BaseModel):
    daily_generations: int = FieldInfo(alias="dailyGenerations")
    """The number of generations that have been run for this model in the past day."""


class UsageModelsGen4Image(BaseModel):
    daily_generations: int = FieldInfo(alias="dailyGenerations")
    """The number of generations that have been run for this model in the past day."""


class UsageModelsGen4ImageTurbo(BaseModel):
    daily_generations: int = FieldInfo(alias="dailyGenerations")
    """The number of generations that have been run for this model in the past day."""


class UsageModelsGen4Turbo(BaseModel):
    daily_generations: int = FieldInfo(alias="dailyGenerations")
    """The number of generations that have been run for this model in the past day."""


class UsageModelsUpscaleV1(BaseModel):
    daily_generations: int = FieldInfo(alias="dailyGenerations")
    """The number of generations that have been run for this model in the past day."""


class UsageModelsVeo3(BaseModel):
    daily_generations: int = FieldInfo(alias="dailyGenerations")
    """The number of generations that have been run for this model in the past day."""


class UsageModelsVeo3_1(BaseModel):
    daily_generations: int = FieldInfo(alias="dailyGenerations")
    """The number of generations that have been run for this model in the past day."""


class UsageModelsVeo3_1Fast(BaseModel):
    daily_generations: int = FieldInfo(alias="dailyGenerations")
    """The number of generations that have been run for this model in the past day."""


class UsageModels(BaseModel):
    act_two: Optional[UsageModelsActTwo] = None
    """Usage data for the act_two model."""

    eleven_multilingual_sts_v2: Optional[UsageModelsElevenMultilingualStsV2] = None
    """Usage data for the eleven_multilingual_sts_v2 model."""

    eleven_multilingual_v2: Optional[UsageModelsElevenMultilingualV2] = None
    """Usage data for the eleven_multilingual_v2 model."""

    eleven_text_to_sound_v2: Optional[UsageModelsElevenTextToSoundV2] = None
    """Usage data for the eleven_text_to_sound_v2 model."""

    eleven_voice_dubbing: Optional[UsageModelsElevenVoiceDubbing] = None
    """Usage data for the eleven_voice_dubbing model."""

    eleven_voice_isolation: Optional[UsageModelsElevenVoiceIsolation] = None
    """Usage data for the eleven_voice_isolation model."""

    gemini_2_5_flash: Optional[UsageModelsGemini2_5Flash] = FieldInfo(alias="gemini_2.5_flash", default=None)
    """Usage data for the gemini_2.5_flash model."""

    gen3a_turbo: Optional[UsageModelsGen3aTurbo] = None
    """Usage data for the gen3a_turbo model."""

    gen4_aleph: Optional[UsageModelsGen4Aleph] = None
    """Usage data for the gen4_aleph model."""

    gen4_image: Optional[UsageModelsGen4Image] = None
    """Usage data for the gen4_image model."""

    gen4_image_turbo: Optional[UsageModelsGen4ImageTurbo] = None
    """Usage data for the gen4_image_turbo model."""

    gen4_turbo: Optional[UsageModelsGen4Turbo] = None
    """Usage data for the gen4_turbo model."""

    upscale_v1: Optional[UsageModelsUpscaleV1] = None
    """Usage data for the upscale_v1 model."""

    veo3: Optional[UsageModelsVeo3] = None
    """Usage data for the veo3 model."""

    veo3_1: Optional[UsageModelsVeo3_1] = FieldInfo(alias="veo3.1", default=None)
    """Usage data for the veo3.1 model."""

    veo3_1_fast: Optional[UsageModelsVeo3_1Fast] = FieldInfo(alias="veo3.1_fast", default=None)
    """Usage data for the veo3.1_fast model."""


class Usage(BaseModel):
    models: UsageModels
    """Usage data for each model."""


class OrganizationRetrieveResponse(BaseModel):
    credit_balance: int = FieldInfo(alias="creditBalance")
    """The number of credits remaining in the organization account."""

    tier: Tier
    """Limits associated with the organization's tier."""

    usage: Usage
    """Usage data for the organization."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["VideoUpscaleCreateParams"]


class VideoUpscaleCreateParams(TypedDict, total=False):
    model: Required[Literal["upscale_v1"]]

    video_uri: Required[Annotated[str, PropertyInfo(alias="videoUri")]]
    """A HTTPS URL."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal

import httpx

from ..types import sound_effect_create_params
from .._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..lib.polling import (
    NewTaskCreatedResponse,
    AsyncNewTaskCreatedResponse,
    create_waitable_resource,
    create_async_waitable_resource,
)
from .._base_client import make_request_options
from ..types.sound_effect_create_response import SoundEffectCreateResponse

__all__ = ["SoundEffectResource", "AsyncSoundEffectResource"]


class SoundEffectResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> SoundEffectResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/runwayml/sdk-python#accessing-raw-response-data-eg-headers
        """
        return SoundEffectResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> SoundEffectResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/runwayml/sdk-python#with_streaming_response
        """
        return SoundEffectResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        model: Literal["eleven_text_to_sound_v2"],
        prompt_text: str,
        duration: float | Omit = omit,
        loop: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> NewTaskCreatedResponse:
        """
        This endpoint will start a new task to generate sound effects from a text
        description.

        Args:
          prompt_text: A text description of the sound effect to generate.

          duration: The duration of the sound effect in seconds, between 0.5 and 30 seconds. If not
              provided, the duration will be determined automatically based on the text
              description.

          loop: Whether the output sound effect should be designed to loop seamlessly.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/v1/sound_effect",
            body=maybe_transform(
                {
                    "model": model,
                    "prompt_text": prompt_text,
                    "duration": duration,
                    "loop": loop,
                },
                sound_effect_create_params.SoundEffectCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=create_waitable_resource(SoundEffectCreateResponse, self._client),
        )


class AsyncSoundEffectResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncSoundEffectResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/runwayml/sdk-python#accessing-raw-response-data-eg-headers
        """
        return AsyncSoundEffectResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncSoundEffectResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/runwayml/sdk-python#with_streaming_response
        """
        return AsyncSoundEffectResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        model: Literal["eleven_text_to_sound_v2"],
        prompt_text: str,
        duration: float | Omit = omit,
        loop: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncNewTaskCreatedResponse:
        """
        This endpoint will start a new task to generate sound effects from a text
        description.

        Args:
          prompt_text: A text description of the sound effect to generate.

          duration: The duration of the sound effect in seconds, between 0.5 and 30 seconds. If not
              provided, the duration will be determined automatically based on the text
              description.

          loop: Whether the output sound effect should be designed to loop seamlessly.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/v1/sound_effect",
            body=await async_maybe_transform(
                {
                    "model": model,
                    "prompt_text": prompt_text,
                    "duration": duration,
                    "loop": loop,
                },
                sound_effect_create_params.SoundEffectCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=create_async_waitable_resource(SoundEffectCreateResponse, self._client),
        )


class SoundEffectResourceWithRawResponse:
    def __init__(self, sound_effect: SoundEffectResource) -> None:
        self._sound_effect = sound_effect

        self.create = to_raw_response_wrapper(
            sound_effect.create,
        )


class AsyncSoundEffectResourceWithRawResponse:
    def __init__(self, sound_effect: AsyncSoundEffectResource) -> None:
        self._sound_effect = sound_effect

        self.create = async_to_raw_response_wrapper(
            sound_effect.create,
        )


class SoundEffectResourceWithStreamingResponse:
    def __init__(self, sound_effect: SoundEffectResource) -> None:
        self._sound_effect = sound_effect

        self.create = to_streamed_response_wrapper(
            sound_effect.create,
        )


class AsyncSoundEffectResourceWithStreamingResponse:
    def __init__(self, sound_effect: AsyncSoundEffectResource) -> None:
        self._sound_effect = sound_effect

        self.create = async_to_streamed_response_wrapper(
            sound_effect.create,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from runwayml import RunwayML, AsyncRunwayML
from tests.utils import assert_matches_type
from runwayml.types import (
    OrganizationRetrieveResponse,
    OrganizationRetrieveUsageResponse,
)
from runwayml._utils import parse_date

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestOrganization:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_method_retrieve(self, client: RunwayML) -> None:
        organization = client.organization.retrieve()
        assert_matches_type(OrganizationRetrieveResponse, organization, path=["response"])

    @parametrize
    def test_raw_response_retrieve(self, client: RunwayML) -> None:
        response = client.organization.with_raw_response.retrieve()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        organization = response.parse()
        assert_matches_type(OrganizationRetrieveResponse, organization, path=["response"])

    @parametrize
    def test_streaming_response_retrieve(self, client: RunwayML) -> None:
        with client.organization.with_streaming_response.retrieve() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            organization = response.parse()
            assert_matches_type(OrganizationRetrieveResponse, organization, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_retrieve_usage(self, client: RunwayML) -> None:
        organization = client.organization.retrieve_usage()
        assert_matches_type(OrganizationRetrieveUsageResponse, organization, path=["response"])

    @parametrize
    def test_method_retrieve_usage_with_all_params(self, client: RunwayML) -> None:
        organization = client.organization.retrieve_usage(
            before_date=parse_date("2019-12-27"),
            start_date=parse_date("2019-12-27"),
        )
        assert_matches_type(OrganizationRetrieveUsageResponse, organization, path=["response"])

    @parametrize
    def test_raw_response_retrieve_usage(self, client: RunwayML) -> None:
        response = client.organization.with_raw_response.retrieve_usage()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        organization = response.parse()
        assert_matches_type(OrganizationRetrieveUsageResponse, organization, path=["response"])

    @parametrize
    def test_streaming_response_retrieve_usage(self, client: RunwayML) -> None:
        with client.organization.with_streaming_response.retrieve_usage() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            organization = response.parse()
            assert_matches_type(OrganizationRetrieveUsageResponse, organization, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncOrganization:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @parametrize
    async def test_method_retrieve(self, async_client: AsyncRunwayML) -> None:
        organization = await async_client.organization.retrieve()
        assert_matches_type(OrganizationRetrieveResponse, organization, path=["response"])

    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncRunwayML) -> None:
        response = await async_client.organization.with_raw_response.retrieve()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        organization = await response.parse()
        assert_matches_type(OrganizationRetrieveResponse, organization, path=["response"])

    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncRunwayML) -> None:
        async with async_client.organization.with_streaming_response.retrieve() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            organization = await response.parse()
            assert_matches_type(OrganizationRetrieveResponse, organization, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_retrieve_usage(self, async_client: AsyncRunwayML) -> None:
        organization = await async_client.organization.retrieve_usage()
        assert_matches_type(OrganizationRetrieveUsageResponse, organization, path=["response"])

    @parametrize
    async def test_method_retrieve_usage_with_all_params(self, async_client: AsyncRunwayML) -> None:
        organization = await async_client.organization.retrieve_usage(
            before_date=parse_date("2019-12-27"),
            start_date=parse_date("2019-12-27"),
        )
        assert_matches_type(OrganizationRetrieveUsageResponse, organization, path=["response"])

    @parametrize
    async def test_raw_response_retrieve_usage(self, async_client: AsyncRunwayML) -> None:
        response = await async_client.organization.with_raw_response.retrieve_usage()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        organization = await response.parse()
        assert_matches_type(OrganizationRetrieveUsageResponse, organization, path=["response"])

    @parametrize
    async def test_streaming_response_retrieve_usage(self, async_client: AsyncRunwayML) -> None:
        async with async_client.organization.with_streaming_response.retrieve_usage() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            organization = await response.parse()
            assert_matches_type(OrganizationRetrieveUsageResponse, organization, path=["response"])

        assert cast(Any, response.is_closed) is True
